"use strict";
this.name      = "Random_Hits"; 
this.author    = "eric walch, spara"; 
this.copyright = "� 2010 Little Bear.";
this.description = "Mission board for generating hit targets. Written by Little Bear and ported to JS by Eric Walch";
this.licence     = "CC BY-NC-SA 3.0";
this.version   = "1.7"; 

// setup some general arrays.
this.spaceBars = ["The Blue Parrot", "The Fox and Hound", "Jon Snorrad's Pub", "Lobbie's Can-Can Lounge", "The Old Spacer", "The Skull and Bones", "The Golden Gargle Blaster", "The Smoke Dragon Lounge", "The Grounded Spacer", "The Finders Keepers Bar", "The Wanted Dead or Alive", "The Marooned Spacer", "The Flying Dutchman", "The Red Pincers", "The Crash and Burn", "Joe's Bar and Grill", "The Bell and Braben", "The Carnal Pleasures Bar", "The Red Light", "The Eject Handle", "The Spacers' Hangout", "The Bearded Parrot", "The Last Wishes", "The Fat Feline", "The Furry Humanoid", "The Slimy Frog", "The Horny Lobster Hotel", "The Pink Slug", "The Colonial's Rest", "The Terminus Hotel", "The Viper and Whistle", "The Habitat Bar", "The Twisted Torus Hotel", "The Blue Lagrange", "O'Neill's Repose", "Giles's Grotto", "The Black Moon", "The Raxxla Ritz Hotel", "The Loving Feline", "The Worlds End", "Seedy Bar"];

this.marks = [
// level 1 marks
["Cobra Mk I", "Gecko", "Krait", "Mamba", "Sidewinder", "Cobra Clipper", "Adder", "Cobra Mk II-X", "Sidewinder Special", "Ophidian Yacht", "Moray Star Boat", "Drake", "Drake Mk II", "DTT Manta", "Aphid", "Bug", "Griffin Mk I"],
// level 2 marks
["Anaconda", "Python", "DTT Mk I", "Boa", "Boa Class Cruiser", "Cobra Mk III", "Fer-de-Lance", "Mussurana", "Python ET Special", "Chameleon", "Ghavial", "Bushmaster", "Monitor", "Asp Mk I", "Cobra Courier", "Cobra Rapier", "Salamander","GalTech Escort Fighter", "Gnat", "Monitor Mk II"],
// level 3 marks
["Asp Mk II",  "Asp Mk II Special", "Cobra Cutlass", "Imperial Courier", "Python Class Cruiser", "Iguana", "Cat Mk I", "Wolf Mk II", "SuperCobra", "Pitviper Mk II", "Vampire Mk II Purgatori", "Imperial Trader", "Purple Haze", "Cobra Mk IV Black", "Cobra S9", "Arachnid"]
];

this.hitMissionEndings = ["COMPLETE", "SCOOPED", "JUMPED", "PODEDOUT", "FAILED", "NPCKILL"];
this.spacebarPositions = [[-5106, 5174, 388275], [12644, 13830, 353962], [25774, -10946, 314064], [-48509, -9524, 266555], [-41190, 33291, 240408]];
this.pages = [{},{},{},{},{},{},{},{},{}]; // create storageroom for 9 hit pages.
this.sets = 3; // Defines how many sets of missions are offered as max. 1 set is 3 contracts. Only 1,2 or 3 sets are allowed.

this.startUp = function (){
    /*
    When the OXP is installed, set the player's score to zero and his title and rank to the lowest 
    Clear all OXP variables in case the player had a test alpha version of the OXP installed
    */
    if (!missionVariables.random_hits_playertitle){
        missionVariables.random_hits_score = 0;
        this.updateStatus();
        delete missionVariables.random_hits_store_showship;
        delete missionVariables.random_hits_fine;
        delete missionVariables.random_hits_npcmoney;
        delete missionVariables.random_hits_markposition;
    } else if (missionVariables.random_hits_mark_galaxy && missionVariables.random_hits_mark_galaxy !== galaxyNumber){
        // player has proceed to new galaxy while the oxp was not installed. Reset all stuff now.
        this.playerEnteredNewGalaxy();        
    }
    
    /*
    A new feature for V1.4 is the player's Criminal Hatred Factor.
    This variable effects wheather 5 Scripted Random Events / Mini-Missions are called and unlocks a special mission.
    On install this is set to zero and the other variables are set up.
    */
    if (!missionVariables.random_hits_criminal_hate_score){
        var inits = ["criminal_hate_score", "events_timer", "events_dice", "special_mission_timer", "dockings_number", 
        "criminal_kill_score", "events_number", "hunter_kill_score", "total_boss_kills", "total_player_boss_kills"];
        for (var i=0; i<inits.length; i++) missionVariables["random_hits_"+inits[i]] = 0;
        missionVariables.random_hits_special_mission = "WAITING"; // yet unused
    }
    if (!missionVariables.random_hits_revenge){
        missionVariables.random_hits_revenge = "NO"; // make sure the variable exists.
        missionVariables.random_hits_striketime = "NOT_NOW";
        missionVariables.random_hits_timer = 0;
    }
    this.firstLaunch = true; // used for setting up space on the first launch. 
};

this.shipDockedWithStation = function (){
    if (player.ship.docked && player.ship.dockedStation.name === "A Seedy Space Bar"){
        // All bars have the same name, only the displayName differs.

        // This point is only reached if docked at a Space Bar. Now we are checking witch pages must be shown.
        // missionScreenOpportunity will fire immediately after this and will deal with the results.
        
        this.setupDockedVariables(); // first define some variables we only needed while docked.
        missionVariables.random_hits_dockings_number++;
        
        // Add a backup mechanism to exit elegant with trunk builds that have a short code limiter setting.
        // This should never trigger in release builds with a limitter value of one second.
        if (this.pagesAreBeingBuild)
        {
            this.hitPageDate = 0;
            log(this.name, "Pages and variables were not set up properly.");
        }
        this.pagesAreBeingBuild = true;
        this.showScreen = "timeOut";

        if(!this.hitPageDate || (this.hitPageDate + 60 < clock.minutes)) this.setupHitpages(); // refresh hitpages, but not within 1 hour.

        this.pendingMessages = [];  // create an empty list. (errases previous lists)
        
        if (player.ship.bounty > 0){
            this.breakStuff(); 
            this.showScreen = "rejectPage";
            this.pagesAreBeingBuild = false; // ready building pages
            return; // no further evaluation!!
        }
        
        this.readHitpage(1); // prepare values for the first page.
        
        // now we have created a message stack and are going to look for events that display a missionscreen.

        if (missionVariables.random_hits_status === "RUNNING"){
            this.pendingMessages.push("busyPageDocking"); // push this name on the pendingMessage stack.
        }

        if (missionVariables.random_hits_status && this.hitMissionEndings.indexOf(missionVariables.random_hits_status) > -1){
            this.pendingMessages.push("missionReport"); // The hit mission is finisched
        }
        
        //Before any can trigger, the player must first qualify for the News Feed Upgrade
        //This is given once the player holds the rank of a Deputy and an Elite rating of Competant and is docked at any bar.
        if (missionVariables.random_hits_score > 10 && player.score > 127){
        
            // add revenge warnings.
            if (missionVariables.random_hits_revenge === "YES" && Math.random() < 0.2){
                this.pendingMessages.push("revengeWarning");
            }
            
            var dice = missionVariables.random_hits_events_dice;
            
            // 1st Type of Random Event - 11% Chance of a group of Super Criminals attacking the Bar
            // If the Chance of a Bar Attack by Super Criminals is Called, show the mission screen asking if the Player wants to join in.
            if (dice > 0 && dice < 12 && !this.bigBossAttack && missionVariables.random_hits_dockings_number > 3){
                this.pendingMessages.push("barAttackStarted");
                missionVariables.random_hits_events_dice = 0;
            }
            
            // test for big bosses and other events now.
            if (this.bigBossAttack){
                this.pendingMessages.push(this.setBigBossMessages()); // setBigBossMessages returns:  "barAttackEnded" or "barAttackInProgress"
            }
            
            // 2nd Type of Random Event - 3% Chance of a increased Thargoid Activity in the System.
            // Extra Thargoids are added to the spacelanes and also near the bar.
            if (dice > 20 && dice < 25 && !this.thargoidAttack && missionVariables.random_hits_dockings_number > 5){
                this.pendingMessages.push("thargoidAttackStarted");
                missionVariables.random_hits_events_dice = 0;
            }

            // test for thargoid events now.
            if (this.thargoidAttack){
                this.pendingMessages.push(this.setThargoidMessages()); // returns:  "barAttackThargoidEnded" or "barAttackThargoidInProgress"
            }
            
            // 3th Type of Random Event - 3% Chance of miningpod theft.
            // Adding ships scooping miningpods.
            if (dice > 30 && dice < 35 && !this.miningpodTheft && missionVariables.random_hits_dockings_number > 5){
                this.pendingMessages.push("miningpodTheftStarted");
                missionVariables.random_hits_events_dice = 0;
            }

            // test for thargoid events now.
            if (this.miningpodTheft){
                this.pendingMessages.push(this.setMiningpodTheftMessages()); // returns:  "miningpodTheftEnded" or "miningpodTheftInProgress"
            }
            
            // Add a welcome message only once for each galaxy (resets after every galactic jump)
            if (!missionVariables.random_hits_has_news_feed){
                this.pendingMessages.push("newsFeedIntro");
                missionVariables.random_hits_has_news_feed = "TRUE";
            }
        }

        if (this.hasNoPendingMessages()){
            this.showScreen = "firstPage"; // default page (used, unless there were pending messages).
        }
        this.pagesAreBeingBuild = false; // ready building pages and messages
        
    }
};

this.missionScreenOpportunity = function (){
    if (player.ship.docked) {
        this.missionOffers();
    }
}

this.shipLaunchedFromStation = function()
{
    if (this.firstLaunch) this.setupShips();
    this.firstLaunch = false;
    delete this.shipLaunchedFromStation; // no longer needed. Bar launching code is now in ship script.
};

this.shipExitedWitchspace = function()
{
    /*
    Increase the timer, but reduce the hate score on each jump.
    Constant hunting therefore keeps the score up, but stopping hunting for a time causes it to fade away.
    */
    this.decreaseCriminalHateBy(1);
    missionVariables.random_hits_events_timer++;
    missionVariables.random_hits_special_mission_timer++;
    missionVariables.random_hits_events_dice = Math.floor(Math.random()*100);
        
    // Add the Space Bar + asteroid cluster + Minesweepers and set its name on entering any Anarchy system
    this.setupShips();
    
    // If the player jumps out leaving a launched pod alive, give a mission failed message and reset the short description.
    if (missionVariables.random_hits_status && missionVariables.random_hits_status === "PODEDOUT"){
		mission.setInstructionsKey("random_hits_shortdescription5");
		player.consoleMessage(expandDescription("[random_hits_failed]"));
		this.$unmarkSystem(missionVariables.random_hits_planetnumber);
		missionVariables.random_hits_status = "FAILED";
    }
    
    if (missionVariables.random_hits_revenge === "YES" && missionVariables.random_hits_criminal_hate_score > 0){
        this.dealWithRevenge();
    }

    // Clear Space Bar name on leaving an Anarchy System
    if (system.government > 0){
		delete missionVariables.random_hits_seedy_spacebar_name;
        delete missionVariables.random_hits_seedy_spacebar_header;
    }
};

this.dealWithRevenge = function (){
    missionVariables.random_hits_timer++;
    // Before the chance of revenge ships appearing is called, clean players get a warning from a helpful bounty hunter within 2 to 5 jumps.
    // If the player is not Clean, no warning is given and the timer for the attack starts running within 2 - 5 jumps anyway.
    if(!missionVariables.random_hits_revenge_warned && (Math.random() > 0.65 || missionVariables.random_hits_timer > 4)){
        missionVariables.random_hits_revenge_warned = "YES";
        if (player.bounty === 0){
            var patrol = system.addShips("random_hits_patrol", 1, [1E3, 1E3, 1E3], 4000);
            if (patrol.length > 0) patrol[0].switchAI("randomhitspatrolwarnAI.plist"); // give it a new AI
        }
        missionVariables.random_hits_timer = 0;
    }
    // Timer now running. There will be a delay before the chance of the ships being added in the spacelanes can be called.
    // If the player is avoiding the spacelanes, the attack ships will eventually jump him by the witchpoint.
    if (missionVariables.random_hits_revenge_warned){
        if (!missionVariables.random_hits_revenge_count) {
            missionVariables.random_hits_revenge_count = Math.ceil(Math.random() * missionVariables.random_hits_criminal_hate_score / 50);
        }
        var shipCount = missionVariables.random_hits_revenge_count;
        if (missionVariables.random_hits_timer > 1 && missionVariables.random_hits_striketime === "SOON" && Math.random() > 0.69){
            this.addRevengeShips(shipCount, Math.random());
            missionVariables.random_hits_striketime = expandDescription("[random_hits_revenge_dice2a]");
        }else if (missionVariables.random_hits_timer > 3 && missionVariables.random_hits_striketime === "QUITE_SOON" && Math.random() > 0.39){
            this.addRevengeShips(shipCount, Math.random());
            missionVariables.random_hits_striketime = expandDescription("[random_hits_revenge_dice2a]");
        }else if (missionVariables.random_hits_timer > 5 && missionVariables.random_hits_striketime === "SLOW_BURN" && Math.random() > 0.29){
            this.addRevengeShips(shipCount, Math.random());
            missionVariables.random_hits_striketime = "JUMP_POINT";
        }
        // If no encounter has occoured within 9 jumps force the timer to call a Jump Point encounter with a 50% chance.
        if (missionVariables.random_hits_timer > 8 && ["SLOW_BURN", "SOON", "QUITE_SOON"].indexOf(missionVariables.random_hits_striketime) > -1 
                            && Math.random() > 0.49){
            this.addRevengeShips(shipCount, 0);
            missionVariables.random_hits_striketime = expandDescription("[random_hits_revenge_dice2]");
        }
        if (missionVariables.random_hits_timer > 2 && missionVariables.random_hits_striketime === "JUMP_POINT" && Math.random() > 0.49){
            this.addRevengeShips(shipCount, 0);
            missionVariables.random_hits_striketime = expandDescription("[random_hits_revenge_dice2]");
        }
        //log(this.name, "Revenge striketime:"+missionVariables.random_hits_striketime+" Revenge jumps: "+missionVariables.random_hits_timer);
    }
};

this.playerEnteredNewGalaxy = function(){
    if (missionVariables.random_hits_status === "RUNNING"){
        player.consoleMessage(expandDescription("[random_hits_abandoned]"));
        this.decreaseCriminalHateBy(this.anyInt(6,8));
        this.updateStatus(); // also deletes destination and mission related variables.
        delete missionVariables.random_hits_has_news_feed; // reset it here so you get the intro message in every galaxy.
    }
};

this.missionOffers = function (){

    // test for completed missions at main stations.
    if (player.ship.dockedStation.isMainStation){
        if (missionVariables.random_hits_status && this.hitMissionEndings.indexOf(missionVariables.random_hits_status) > -1){
            this.showScreen = "missionReport"
        }
        // setup the page display sequence for the handom hits station.
        if (this.showScreen) switch (this.showScreen){
            case "missionReport": {
                missionVariables.random_hits_security_bar_text_close = expandDescription("[random_hits_security_bar_text_close_extra4-0]");
                this.reportMissionEnded();
                break;
            }
            case "freepod": {
                mission.runScreen({title: "Random Hits", messageKey: "random_hits_freepod", music: "randomhitsmail.ogg",
                                    model: "police", choicesKey: "random_hits_pause4"},
                                        this.choiceEvaluation);
                this.decreaseCriminalHateBy(this.anyInt(6,10));
                player.credits += missionVariables.random_hits_galcop;
                this.updateStatus();
                break;
            }
            case "killedTarget": {
                mission.runScreen({title: "Random Hits", messageKey: "random_hits_killpod", music: "randomhitsmail.ogg",
                                    model: "random_hits_gravemarker", choicesKey: "random_hits_pause"},
                                    this.choiceEvaluation);
                missionVariables.random_hits_criminal_hate_score += this.anyInt(6,10);
                player.credits += missionVariables.random_hits_store_mark_fee;
                this.testRevenge();
                this.updateStatus();
                break;
            }
            default:  break;
         }
    } 

    // Docked at one of the spacebars ...
    if (this.random_hits_docked){
        //Check if the Player has more than one message waiting. If so the Second Message Display will wait until the 1st one has been read

        // setup the page display sequence for the handom hits station.
        if (this.showScreen) switch (this.showScreen){
            case "firstPage":
            case "secondPage":
            case "thirdPage": {		
                if (++this.hitpageNumber > this.hitpageMaxNumber) this.hitpageNumber = 1;
                this.targetLevel = modulo(this.hitpageNumber-1, 3)+1; // returns 1, 2 or 3
                this.readHitpage(this.hitpageNumber);
                var page = "level_"+this.targetLevel+"_mark_advert";
                mission.runScreen({title: "Random Hits Target Board", messageKey: page,
                                    model: this.getMarkRole(missionVariables.random_hits_mark_ship),
                                    choicesKey: "random_hits_board_choice"+this.targetLevel},
                                    this.choiceEvaluation);
                break;
            }
            case "barBill": {
                this.evaluateRevengeStatus();
                var message = expandMissionText("random_hits_barbill", {random_hits_billdue: this.random_hits_billdue});
                mission.runScreen({title: "Random Hits Bar", message: message, music: "randomhitsmail.ogg",
                                    model: "random_hits_griff_spacebar", choicesKey: "random_hits_bill_pause"},
                                    this.choiceEvaluation);
                mission.displayModel.orientation = [0.707,-0.707,0,0];
                player.credits -= this.random_hits_billdue;
                break;
            }
            case "noExperience": {
                mission.runScreen({title: "Random Hits Target Board", messageKey: "random_hits_refused", music: "randomhitsmail.ogg",
                                    model: this.getMarkRole(missionVariables.random_hits_mark_ship),
                                    choicesKey: "random_hits_board_choice0"},
                                    this.choiceEvaluation);
                break;
            }
            case "markAccepted": {
                this.setStoreVariables(this.hitpageNumber);
                mission.runScreen({title: "Random Hits Target Board", messageKey: "random_hits_jobaccepted", music: "randomhitsmail.ogg",
                                    model: this.getMarkRole(missionVariables.random_hits_mark_ship),
                                    choicesKey: "random_hits_board_choice4"},
                                    this.choiceEvaluation);
                this.$markSystem(missionVariables.random_hits_planetnumber);
                mission.setInstructionsKey("random_hits_shortdescription");
                this.setupMarkPosition(); // sets mark position and various fees.
                missionVariables.random_hits_status ="RUNNING";
                break;
            }
            case "busyPage": {
                if (missionVariables.random_hits_store_mark_fee/10 < player.credits) {
                mission.runScreen({title: "Random Hits Current Mission Target", messageKey: "random_hits_busy", music: "randomhitsmail.ogg",
                                    model: this.getMarkRole(missionVariables.random_hits_store_mark_ship),
                                    choicesKey: "random_hits_board_choice5"},
                                    this.choiceEvaluation);
                } else {
                mission.runScreen({title: "Random Hits Current Mission Target", messageKey: "random_hits_busy", music: "randomhitsmail.ogg",
                                    model: this.getMarkRole(missionVariables.random_hits_store_mark_ship),
                                    choicesKey: "random_hits_board_choice5_no_cancel"},
                                    this.choiceEvaluation);
                }
                break;
            }
            case "busyPageDocking": {
                if (missionVariables.random_hits_store_mark_fee/10 < player.credits) {
                mission.runScreen({title: "Random Hits Current Mission Target", messageKey: "random_hits_busy_docking", music: "randomhitsmail.ogg",
                                    model: this.getMarkRole(missionVariables.random_hits_store_mark_ship),
                                    choicesKey: "random_hits_board_choice6"},
                                    this.choiceEvaluation);
                } else {
                mission.runScreen({title: "Random Hits Current Mission Target", messageKey: "random_hits_busy_docking", music: "randomhitsmail.ogg",
                                    model: this.getMarkRole(missionVariables.random_hits_store_mark_ship),
                                    choicesKey: "random_hits_board_choice6_no_cancel"},
                                    this.choiceEvaluation);
                }
                break;
            }
            case "rejectPage": {
                mission.runScreen({title: "Random Hits Bar", messageKey: "random_hits_notwelcome",
                                    choicesKey: "random_hits_eject"},
                                    this.choiceEvaluation);
                break;
            }
            case "newsFeedIntro" : {
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_news_feed_award", model: "random_hits_griff_spacebar",
                                    choicesKey: "random_hits_pause6", music: "randomhitsmail.ogg"},
                                    this.choiceEvaluation);
                break;
            }
            case "missionReport": {
                this.reportMissionEnded();
                break;
            }
            case "freepod": {
                mission.runScreen({title: "Random Hits", messageKey: "random_hits_freepod", music: "randomhitsmail.ogg",
                                    model: "police", choicesKey: "random_hits_pause4"},
                                        this.choiceEvaluation);
                missionVariables.random_hits_criminal_hate_score += this.anyInt(6,10);
                player.credits += missionVariables.random_hits_galcop;
                this.updateStatus();
                break;
            }
            case "killedTarget": {
                mission.runScreen({title: "Random Hits", messageKey: "random_hits_killpod", music: "randomhitsmail.ogg",
                                    model: "random_hits_gravemarker", choicesKey: "random_hits_pause_goto_board"},
                                    this.choiceEvaluation);
                missionVariables.random_hits_criminal_hate_score += this.anyInt(6,10);
                player.credits += missionVariables.random_hits_store_mark_fee;
                this.testRevenge();
                this.updateStatus();
                break;
            }
            case "barAttackStarted" : {
                missionVariables.random_hits_reward_offer_name = expandDescription("[random_hits_security_bar_text9]");;
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_big_boss_alert", model: "random_hits_big_boss",
                                    choicesKey: "random_hits_big_boss_alert_first_choice", music: "siren.ogg"},
                                    this.choiceEvaluation);
				missionVariables.random_hits_dockings_number = 0;
				missionVariables.random_hits_criminal_kill_score = 0;
				missionVariables.random_hits_hunter_kill_score = 0;
				missionVariables.random_hits_total_boss_kills = 0;
				missionVariables.random_hits_total_player_boss_kills = 0;
				missionVariables.random_hits_events_number++;
				missionVariables.random_hits_battle_time = clock.days;
				missionVariables.random_hits_battle_system = system.info.name;
				missionVariables.random_hits_battle_bar = player.ship.dockedStation.displayName;
                var pos = player.ship.dockedStation.position;
                var bossCount = this.anyInt(3,6);
                missionVariables.random_hits_battle_boss_number = bossCount;
                missionVariables.random_hits_battle_fighter_number = 6;
                system.addGroup("random_hits_big_boss", bossCount, pos, 10000);
                system.addGroup("random_hits_big_boss_fighter", 6, pos, 10000);
                this.bigBossAttack = true;
                break;
            }
            case "barAttackInProgress" : {
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_big_boss_alert2",
                                    model: "random_hits_big_boss",
                                    choicesKey: "random_hits_big_boss_alert_second_choice", music: "siren.ogg"},
                                    this.choiceEvaluation);
				player.credits += missionVariables.random_hits_criminal_kill_payment;
                this.clearBigBossDescriptions();
               break;
            }
            case "barAttackEnded" : {
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_big_boss_alert3",
                                    model: "random_hits_gravemarker",
                                    choicesKey: "random_hits_pause5", music: "randomhitsmail.ogg"},
                                    this.choiceEvaluation);
				player.credits += missionVariables.random_hits_criminal_kill_payment;
                this.clearBigBossDescriptions();
				delete missionVariables.random_hits_battle_time;
				delete missionVariables.random_hits_battle_system;
                delete missionVariables.random_hits_battle_bar;
				delete missionVariables.random_hits_describe_total_player_kills;
				delete missionVariables.random_hits_describe_prize;
				missionVariables.random_hits_dockings_number = 0;
				missionVariables.random_hits_hunter_kill_score = 0;
				missionVariables.random_hits_total_boss_kills = 0;
				missionVariables.random_hits_total_player_boss_kills = 0;		
                this.bigBossAttack = false;
               break;
            }
            case "thargoidAttackStarted": {
                var pos = player.ship.dockedStation.position;
                // first add a dummy message to prevent the player from looking at a black screen, because adding thargoids takes time
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_pause_message", music: "siren.ogg"});
                missionVariables.random_hits_player_thargoid_kills = 0;
                missionVariables.random_hits_thargoid_reward = this.anyRoundedInt(500,1600);  //  check this value. to low/to high ??
                // the new ship adding commands return a group. Use this group as parameter to setup the rewards for that group.
                this.setUpThargoidRewards(system.addGroup("thargoid", this.anyInt(3,6), pos, 10000));
                this.setUpThargoidRewards(system.addGroup("thargoid", this.anyInt(3,6), pos, 30000));
                mission.addMessageText("\n\n              nearly done.....");
                if (player.ship.dockedStation.minesweepers.count > 1) {
                    // scans for explosive tharglets. 
                    player.ship.dockedStation.minesweepers.ships[1].AIState = "LOOK_FOR_TARGOIDS";
                }
                for (var i=1; i<5; i++) this.setUpThargoidRewards(system.addGroupToRoute("thargoid", 1, 0.2*i, "wp"));
                missionVariables.random_hits_thargoid_number = system.countShipsWithPrimaryRole("thargoid");
                missionVariables.random_hits_thargoid_number_close = system.shipsWithPrimaryRole("thargoid", player.ship.dockedStation, 25E3).length;
                missionVariables.random_hits_reward_offer_name = expandDescription("[random_hits_security_bar_text9]");
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_thargoid_spacelane_alert", model: "thargoid",
                                    choicesKey: "random_hits_thargoid_alert_first_choice", music: "siren.ogg"},
                                    this.choiceEvaluation);
				this.thargoidAttack = true;
				missionVariables.random_hits_dockings_number = 0;
				missionVariables.random_hits_events_number++;
                break;
            }
            case "barThargoidAttackInProgress": {
                missionVariables.random_hits_reward_offer_name = expandDescription("[random_hits_security_bar_text9]");
                missionVariables.random_hits_thargoid_total_reward = 
                                missionVariables.random_hits_thargoid_reward * missionVariables.random_hits_player_thargoid_kills;
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_thargoid_spacelane_alert_progressing", model: "thargoid",
                                    choicesKey: "random_hits_thargoid_alert_first_choice", music: "siren.ogg"},
                                    this.choiceEvaluation);
                missionVariables.random_hits_player_thargoid_kills = 0;
                player.credits += missionVariables.random_hits_thargoid_total_reward;
                missionVariables.random_hits_thargoid_total_reward = 0;
                break;
            }
            case "barThargoidAttackEnded": {
                missionVariables.random_hits_reward_offer_name = expandDescription("[random_hits_security_bar_text9]");
                missionVariables.random_hits_thargoid_total_reward = 
                                missionVariables.random_hits_thargoid_reward * missionVariables.random_hits_player_thargoid_kills;
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_thargoid_spacelane_alert_over", model: "random_hits_gravemarker",
                                    choicesKey: "random_hits_pause5", music: "randomhitsmail.ogg"},
                                    this.choiceEvaluation);
                player.credits += missionVariables.random_hits_thargoid_total_reward;
                delete missionVariables.random_hits_player_thargoid_kills;
                delete missionVariables.random_hits_thargoid_number_close;
                delete missionVariables.random_hits_thargoid_number;
                delete missionVariables.random_hits_thargoid_reward;
                delete missionVariables.random_hits_thargoid_total_reward;
                break;
            }
            case "miningpodTheftStarted": {
                var pos = player.ship.dockedStation.position;
                missionVariables.random_hits_miningpodThief_reward = this.anyRoundedInt(300,800);  //  check this value. to low/to high ??
                missionVariables.random_hits_miningpodThief_number = this.setUpMiningpodThief(system.addGroup("pirate", this.anyInt(5,10), pos, 35000));
                missionVariables.random_hits_miningpodThief_player_kill = 0;
                missionVariables.random_hits_miningpodThief_groupname = expandDescription("[random_hits_podThief_alert_subject]");
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_miningpodThief", model: "random_hits_mining_pod",
                                    choicesKey: "random_hits_miningpod_thief_first_choice", music: "siren.ogg"},
                                    this.choiceEvaluation);
				this.miningpodTheft = true;
				missionVariables.random_hits_dockings_number = 0;
				missionVariables.random_hits_events_number++;
                break;
            }
            case "miningpodTheftInProgress": {
                missionVariables.random_hits_miningpodThief_total_reward = missionVariables.random_hits_miningpodThief_player_kill
                                        * missionVariables.random_hits_miningpodThief_reward;
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_miningpodThief_progressing", model: "random_hits_mining_pod",
                                    choicesKey: "random_hits_miningpod_thief_first_choice", music: "siren.ogg"},
                                    this.choiceEvaluation);
                player.credits += missionVariables.random_hits_miningpodThief_total_reward;
                missionVariables.random_hits_miningpodThief_player_kill = 0;
                break;
            }
            case "miningpodTheftEnded": {
                missionVariables.random_hits_miningpodThief_total_reward = missionVariables.random_hits_miningpodThief_player_kill
                                        * missionVariables.random_hits_miningpodThief_reward;
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_miningpodThief_over",
                                    model: "random_hits_griff_autominer", choicesKey: "random_hits_pause5", music: "randomhitsmail.ogg"},
                                    this.choiceEvaluation);
                player.credits += missionVariables.random_hits_miningpodThief_total_reward;
                delete missionVariables.random_hits_miningpodThief_total_reward;
                delete missionVariables.random_hits_miningpodThief_player_kill;
                delete missionVariables.random_hits_miningpodThief_reward;
                delete missionVariables.random_hits_miningpodThief_number;
                delete missionVariables.random_hits_miningpodThief_groupname;
                break;
            }
            case "revengeWarning": {
                mission.runScreen({title: "Random Hits News", messageKey: "random_hits_revenge_alert",
                                    choicesKey: "random_hits_pause5", music: "randomhitsmail.ogg"},
                                    this.choiceEvaluation);
                break;
            }
            case "cancelMission": {
                this.decreaseCriminalHateBy(this.anyInt(6,10));
                missionVariables.random_hits_store_mark_fee /= 10;
                player.credits -=  missionVariables.random_hits_store_mark_fee;
                missionVariables.random_hits_score -= missionVariables.random_hits_store_skill;
                mission.runScreen({title: "Random Hits", messageKey: "random_hits_cancelledMission", music: "randomhitsmail.ogg",
                                    model: "random_hits_gravemarker", choicesKey: "random_hits_pause7"},
                                    this.choiceEvaluation);
                this.testRevenge();
                this.updateStatus();
                break;
            }
            case "timeOut": {
                mission.runScreen({title: "Random Hits", messageKey: "random_hits_slowComputer", model: "random_hits_gravemarker"});
                log(this.name, "Succeeded in generating "+this.hitpageMaxNumber
                    + " hitpages. You should only see this message with Oolite debug builds or in release builds at slow computers.");
                missionVariables.random_hits_assassination_board_address1 = expandDescription("[assassination_board_address1-1]");
                missionVariables.random_hits_assassination_board_address2 = expandDescription("[assassination_board_address2-1]");
                this.showScreen = "barBill";
                break;
            }

            default: {
                log(this.name,"Error **** Got request to show unknow page: "+this.showScreen);
                break;
            }
        }
    }
};

this.choiceEvaluation = function (choice){
    switch(choice){
        case "random_hits_bill_paid": {// do nothing (random_hits_bill_pause)
            if (hasNoPendingMessages()) this.showScreen = false; // no more pages to display.
            break;
        }
        case "random_hits_accept": { //  (random_hits_board_choice1, random_hits_board_choice2, random_hits_board_choice3)
            this.showScreen = "markAccepted";
            if ((this.targetLevel === 1 && player.score < 32) || (this.targetLevel === 2 && player.score < 128)
                                         || (this.targetLevel === 3 && player.score < 512)) this.showScreen = "noExperience";
            if (missionVariables.random_hits_status === "RUNNING") this.showScreen = "busyPage";
            break;
        }

        case "random_hits_2nd_hit": { //  (random_hits_board_choice1)
            this.showScreen = "secondPage";
            break;
        }

        case "random_hits_3th_hit": { //  (random_hits_board_choice2)
            this.showScreen = "thirdPage";
            break;
        }

        case "random_hits_paidplayer": // (random_hits_pause)
        case "random_hits_failedplayer": // (random_hits_pause2)
        case "random_hits_failedbynpc": // (random_hits_pause3)
        case "random_hits_big_boss_battle_over": //  (random_hits_pause5)
        case "random_hits_goto_board": //(random_hits_pause_goto_board)
        case "random_hits_returnwithjob":  //  (random_hits_board_choice4)
        case "random_hits_refusedreturn": //  (random_hits_board_choice0)
        case "random_hits_returnbusy": // (random_hits_board_choice5)
        case "random_hits_news_awarded": // (random_hits_pause6)
        case "random_hits_1st_hit": { //  (random_hits_board_choice3)
            // test for pending messages. If not found, show the first screen
            if (hasNoPendingMessages()) this.showScreen = "firstPage"; // NB the main station has no "firstPage" and just skips this.
            break;
        }

        case "random_hits_news_awarded":
        case "random_hits_failedself": // (random_hits_pause4)
        case "random_hits_big_boss_alert_first_choice_notnow": // (random_hits_big_boss_alert_first_choice)
        case "random_hits_big_boss_alert_second_choice_notnow": //(random_hits_big_boss_alert_second_choice)
        case "random_hits_thargoid_alert_first_choice_notnow": //(random_hits_thargoid_alert_second_choice)
        case "random_hits_zexitwithjob": // (random_hits_board_choice4)
        case "random_hits_zrefusedexit": // (random_hits_board_choice0)
        case "random_hits_zexitbusy": // (random_hits_board_choice5)
        case "random_hits_zexit": { //  (random_hits_board_choice1, random_hits_board_choice2, random_hits_board_choice3)
            // test for pending messages. Else show the bill
            if (hasNoPendingMessages()) this.showScreen = "barBill";
            break;
        }

        case "random_hits_live": { // (random_hits_livedie)
            // choose a new board poster name for this one.
            missionVariables.random_hits_store_assassination_board_poster_title = expandDescription("[assassination_board_poster_name]");
            this.showScreen = "freepod";
            delete missionVariables.random_hits_status;
            break;
        }
        case "random_hits_murder":{ // (random_hits_livedie)
            this.showScreen = "killedTarget";
            delete missionVariables.random_hits_status;
            break;
        }
        case "random_hits_kickout":{ // (random_hits_notwelcome)
            this.decreaseCriminalHateBy(this.anyInt(6,10));
            this.launchPlayerFromBar();
            break;
        }
        case "random_hits_big_boss_alert_first_choice_launch":{ // (random_hits_big_boss_alert_first_choice)
            missionVariables.random_hits_criminal_hate_score++;
            var pos = player.ship.dockedStation.position;
            var bossCount = this.anyInt(1,3);
            missionVariables.random_hits_battle_boss_number += bossCount;
            system.addGroup("random_hits_big_boss", bossCount, pos, 8000);
            this.launchPlayerFromBar();
            break;
        }
        case "random_hits_miningpod_thief_first_choice_launch":
        case "random_hits_thargoid_alert_first_choice_launch":
        case "random_hits_big_boss_alert_second_choice_launch":{
            this.launchPlayerFromBar();
            break;
        }
        case "random_hits_the_cancellation":{ // (random_hits_board_choice6)
            this.showScreen = "cancelMission";
            break;
        }

        default: log(this.name,"Got unknown mission choice: "+choice);
    }
};

// Process the mission ended in a seperate routine because it is handled a bit different for main stations and seedy bars.
// In the bars it is preceeded by an incoming mail message. 

this.reportMissionEnded = function (){
    switch(missionVariables.random_hits_status){
        case "COMPLETE": {
                mission.runScreen({title: "Random Hits Mission Completed", messageKey: "random_hits_paid",
                                    music: "randomhitsmail.ogg", model: "random_hits_gravemarker",
                                    choicesKey: "random_hits_pause"},
                                    this.choiceEvaluation);
                this.testRevenge();
                player.credits +=  missionVariables.random_hits_store_mark_fee;
                if (!missionVariables.random_hits_criminal){
                    missionVariables.random_hits_score += missionVariables.random_hits_store_skill;
                }
                this.updateStatus(); // clears also several other stored mission variables and destination system..
                break;
        }
        case "SCOOPED": {
                mission.runScreen({title: "Random Hits Mission Completed", messageKey: "random_hits_markscooped",
                                    music: "randomhitsmail.ogg", model: missionVariables.random_hits_store_showship,
                                    choicesKey: "random_hits_livedie"},
                                    this.choiceEvaluation);
                if (!missionVariables.random_hits_criminal){
                    missionVariables.random_hits_score += missionVariables.random_hits_store_skill;
                // no update status here. Do it after the choice.
                 }
                break;
        }
        case "JUMPED": {
                mission.runScreen({title: "Random Hits Mission Failed", messageKey: "random_hits_jumpfail",
                                    music: "randomhitsmail.ogg", model: missionVariables.random_hits_store_showship,
                                    choicesKey: "random_hits_pause2"},
                                    this.choiceEvaluation);
                this.updateStatus(); // clears also several other stored mission variables.
                break;
        }
        case "PODEDOUT": this.removeEscapePod();
        case "FAILED": {
                mission.runScreen({title: "Random Hits Mission Failed", messageKey: "random_hits_failed",
                                    music: "randomhitsmail.ogg", model: missionVariables.random_hits_store_showship,
                                    choicesKey: "random_hits_pause2"},
                                    this.choiceEvaluation);
                this.updateStatus(); // clears also several other stored mission variables.
                break;
        }
        case "NPCKILL": {
                mission.runScreen({title: "Random Hits Mission Failed", messageKey: "random_hits_npckill",
                                    music: "randomhitsmail.ogg", model: "random_hits_gravemarker",
                                    choicesKey: "random_hits_pause3"},
                                    this.choiceEvaluation);
                player.credits +=  missionVariables.random_hits_npcmoney;
                this.updateStatus(); // clears also several other stored mission variables.
                break;
        }
        default: log(this.name, "***** Found an unknown key during missionreport: "+missionVariables.random_hits_status);
                break;
    }
};

this.removeEscapePod = function () {
  // TODO
};

this.launchPlayerFromBar = function (){
    this.pendingMessages = [];  // empty the message list.
    player.ship.launch();
}

this.setupShips = function (){
			if (system.isInterstellarSpace) return;
			//If the player has a mission running, check that he is at the victim's system.
			if (missionVariables.random_hits_status === "RUNNING" && missionVariables.random_hits_planetnumber === system.ID){
				player.consoleMessage(expandDescription("[random_hits_intro]"), 12);
        
				//  add the mark's ship somewhere in the spacelanes.
				system.addShipsToRoute(this.getMarkRole(missionVariables.random_hits_store_mark_ship), 1, missionVariables.random_hits_markposition/100);
			}			
			if (system.government === 0){
				var barNumber = Math.floor(system.ID/6.4); // generates 0...39 (bar nr 1...40)
				var barPosition = new Vector3D(this.spacebarPositions[this.modulo(barNumber,5)]); // generates 0...4
				barNumber++;
				
				var validatorBarRole = "random_hits_spacebar" + barNumber;
				if (worldScripts.station_validator && (worldScripts.station_validator.$deaths(validatorBarRole).length !== 0))
					return;
				
				barPosition = system.mainPlanet.position.subtract(barPosition); // translate pwu into wpu
				
				this.setupSpacebar(barNumber, barPosition);
				
				// Random Bar Attacks
				if (Math.random() < 0.15){
					system.addGroup("random_hits_big_boss", this.anyInt(3,6), barPosition, 10000);
					system.addGroup("random_hits_big_boss_fighter", 6, barPosition, 10000);
				}
			}
};

this.addRevengeShips = function (count, pos){
    system.addGroupToRoute("random_hits_mark_revenge", count, pos, "wp");
    missionVariables.random_hits_timer = 0;
    // log(this.name, "Added "+count+" revengeship at position "+pos+" from witchpont");
};

this.setupSpacebar = function (barNumber, barPosition){
    var barRole = "random_hits_spacebar" + barNumber;
    var barName = this.spaceBars[barNumber - 1];
    missionVariables.random_hits_seedy_spacebar_header = this.barDescription(barName);
    var spacebar = system.addShips(barRole, 1, barPosition, 10)[0];
    spacebar.displayName = barName;
    system.addShips("random_hits_griff_autominer", this.anyInt(7,12), barPosition, 15000);
    spacebar.minesweepers = new ShipGroup("Minesweepers");
    spacebar.minesweepers.leader = spacebar;
    system.addShips("asteroid", this.anyInt(6,9), barPosition, 25000);
    var minesweeper;
    for (var i = 0; i<3; i++){
        minesweeper = system.addShips("random_hits_minesweeper", 1, barPosition.add(new Vector3D.randomDirection(8000)), 5)[0];
        spacebar.minesweepers.addShip(minesweeper);
        minesweeper.group = spacebar.minesweepers;
    }
    
	function allShips(entity) {return entity.isShip}; 
	var problem = system.filteredEntities(this, allShips, spacebar, 5E3); // look for any nearby ships during spawning.
    for (var i=0; i< problem.length; i++){
        // rocks, added to close to the bar can block launches. move them a bit outward from the bar.
        var direction = problem[i].position.subtract(spacebar.position).direction();
        problem[i].position = problem[i].position.add(direction.multiply(5E3)); // place 5000 m further away.
    }
    spacebar.orientation = new Quaternion(100, 100, 0, system.ID).normalize(); // always the same in a system.
    if (0 >= oolite.compareVersion("1.77")){
        spacebar.setInterface("RandomHits",
            { title: "Random Hits",
            category: "Contracts",
            summary: "Enter Random Hits Target Board.",
            callback: this.$restartHitPages.bind(this) });
    }
};

this.barDescription = function (name){
    // create a centered barname, independent of the used font.
    var barTitle = " You are viewing " + name + " Bulletin Board. ";
    var titleSize = defaultFont.measureString(barTitle); // missionscreen width = 32 em
    var starSize = defaultFont.measureString("**");
    var stars = "******************************".substr(0, Math.floor((32 - titleSize)/starSize));
    barTitle = stars + barTitle + stars;
    if ((32 - defaultFont.measureString(barTitle)) > starSize/2) barTitle = barTitle + "*";
    return barTitle;
}

this.anyInt = function (min, max){
    return Math.floor(Math.random()*(1+max-min))+min;
};

this.anyRoundedInt = function (min, max){
    return Math.floor(this.anyInt(min, max)/5)*5;
};

this.modulo = function (a,b){
    return Math.round(((a/b) - Math.floor(a/b))*b);
};

this.hasNoPendingMessages = function (){
    var result = true;
    var count = 0;
    var news = "";
    if (this.pendingMessages){
        var count = this.pendingMessages.length;
        if (count > 0){
            this.showScreen = this.pendingMessages.pop(); // get the last message
            result = false;
            count--;
        }
    }
    news = expandDescription("[random_hits_security_bar_text_close_extra4-"+count+"]");
    missionVariables.random_hits_security_bar_text_close = news; //set the botom line
    return result;
};

this.setStoreVariables = function (nr){
    missionVariables.random_hits_planetnumber = System.systemIDForName(missionVariables.random_hits_mark_system);
    missionVariables.random_hits_mark_galaxy = galaxyNumber;
    missionVariables.random_hits_show_galaxy = galaxyNumber+1;
    var transfers = ["assassination_board_poster_title", "assassination_board_job_name", "assassination_board_poster_name", 
    "assassination_board_poster_system", "assassination_board_subject", "assassination_board_address1", "assassination_board_address2", 
    "assassination_board_part1", "mark_first_name", "mark_nick_name", "mark_second_name", "mark_race_part1", "mark_race_part2",
    "assassination_board_part2", "assassination_board_part3", "mark_ship_description", "mark_ship", "mark_ship_name", "mark_ship_ad_name", "assassination_board_part4", "mark_system", "assassination_board_part5", "mark_fee", "assassination_board_part6", "mark_gender", "assassination_board_part7"];
    for (var i=0; i<transfers.length; i++) missionVariables["random_hits_store_"+transfers[i]] = missionVariables["random_hits_"+transfers[i]];
    missionVariables.random_hits_store_skill = nr;
    missionVariables.random_hits_criminal_hate_score += (this.anyInt(6,10) + nr);
};

this.resetStoreVariables = function (){
    var deletions = ["planetnumber", "mark_galaxy", "show_galaxy", "galcop", "store_assassination_board_poster_title",
    "store_assassination_board_job_name", "store_assassination_board_poster_name", "store_assassination_board_poster_system",
    "store_assassination_board_subject", "store_assassination_board_address1", "store_assassination_board_address2", 
    "store_assassination_board_part1", "store_mark_first_name", "store_mark_nick_name", "store_mark_second_name", "store_mark_race_part1", 
    "store_mark_race_part2", "store_assassination_board_part2", "store_assassination_board_part3", "store_mark_ship_description", 
    "store_mark_ship", "store_mark_ship_name", "store_mark_ship_ad_name", "store_assassination_board_part4", "store_mark_system", "store_assassination_board_part5", "store_mark_fee", "store_assassination_board_part6", "store_mark_gender", "store_assassination_board_part7", "store_skill", "deathcause", "npcdays", 
    "npcmoney", "store_showship", "markposition", "fine"];
    for (var i=0; i<deletions.length; i++) delete missionVariables["random_hits_"+deletions[i]];
};

// Break stuff when docking at a bar while not clean. We don't use a predefined list but select equipment from
// any equipment a player carries. Could be oxp added equipment.
this.breakStuff = function (){
    var myEquipmentList = player.ship.equipment;
    var myEquipment = myEquipmentList[Math.floor(myEquipmentList.length * Math.random())];
    var myName = myEquipment.name;
    // log(this.name, "Evaluate to break a: "+myName);
    if (myEquipment.effectiveTechLevel < Math.random() * 15 && player.ship.equipmentStatus(myEquipment.equipmentKey) === "EQUIPMENT_OK"){
        player.ship.setEquipmentStatus(myEquipment.equipmentKey, "EQUIPMENT_DAMAGED")
        myName = expandDescription("[random_hits_notwelcome_damaged] "+myName);
    } else {
        myName = expandDescription("[random_hits_nothing]");
    }
    missionVariables.random_hits_broken = myName;
};

this.setupMarkPosition = function (){
    // set the mark's position, the fine for using an e-bomb or q-mine and the consolation payment 
    // if an npc kills the mark and payment for taking him alive
    missionVariables.random_hits_npcmoney = this.anyRoundedInt(55,270);
    missionVariables.random_hits_markposition = this.anyInt(10,90);
    var skill = missionVariables.random_hits_store_skill;
    missionVariables.random_hits_fine = 1000 * skill + this.anyRoundedInt(300,995);
    var lowScore = skill===1 ? 250 : (skill===2 ? 1250 : 5250);
    var highScore = skill===1 ? 850 : (skill===2 ? 6325 : 9325);
    missionVariables.random_hits_galcop = this.anyRoundedInt(lowScore, highScore);
}

this.getMarkRole = function (shipName){
    // Take a ship's name and return its role.
    var role;
    var mark = "random_hits_mark_";
    switch(shipName){
		case "Aphid":           role = mark + "aphid"; break;
        case "Sidewinder Special":           role = mark + "sidewinderx"; break;
        case "Cobra Mk I":          role = mark + "cobra1"; break;
        case "Gecko":               role = mark + "gecko"; break;
        case "Krait":               role = mark + "krait"; break;
        case "Mamba":               role = mark + "mamba"; break;
        case "Sidewinder":          role = mark + "sidewinder"; break;
        case "Cobra Clipper":         role = mark + "cobraclipper"; break;
        case "Adder":               role = mark + "adder"; break;
        case "Cobra Mk II-X":       role = mark + "cobra2"; break;
        case "Ophidian Yacht":      role = mark + "yacht"; break;

        case "Moray Star Boat":     role = mark + "moray"; break;
        case "Drake":               role = mark + "drake"; break;
        case "Drake Mk II":         role = mark + "drake2"; break;
        case "DTT Manta":      role = mark + "manta"; break;
        case "Anaconda":            role = mark + "anaconda"; break;
        case "Python":              role = mark + "python"; break;
        case "Boa":                 role = mark + "boa"; break;
        case "Boa Class Cruiser":   role = mark + "boa2"; break;
        case "Cobra Mk III":        role = mark + "cobra3"; break;
        case "Fer-de-Lance":        role = mark + "ferdelance"; break;

        case "Mussurana":           role = mark + "mussurana"; break;
        case "Python ET Special":   role = mark + "pythonet"; break;
        case "Chameleon":           role = mark + "chameleon"; break;
        case "Ghavial":             role = mark + "ghavial"; break;
        case "Bushmaster":          role = mark + "bushmaster"; break;
        case "Monitor":             role = mark + "monitor"; break;
        case "DTT Mk I":     role = mark + "dttmk1"; break;
        case "Asp Mk I":            role = mark + "aspmk1"; break;
        case "Cobra Courier":       role = mark + "cobra3courier"; break;
        case "Cobra Rapier":        role = mark + "cobra3rapier"; break;

        case "Salamander":          role = mark + "salamander"; break;
        case "GalTech Escort Fighter":          role = mark + "galtech_escort_fighter"; break;
        case "Asp Mk II":           role = mark + "asp"; break;
        case "Asp Mk II Special":   role = mark + "aspspec"; break;
        case "Cobra Cutlass":        role = mark + "cutlass"; break;
        case "Imperial Courier":    role = mark + "imp"; break;
        case "Python Class Cruiser": role = mark + "pcc"; break;
        case "Iguana":              role = mark + "igu"; break;
        case "Cat Mk I":            role = mark + "cat"; break;
        case "Wolf Mk II":          role = mark + "wolf"; break;

        case "Purple Haze":         role = mark + "phaze"; break;
        case "SuperCobra":         role = mark + "supercobra"; break;
        case "Pitviper Mk II":          role = mark + "pitviper"; break;
        case "Vampire Mk II Purgatori":      role = mark + "vamppurg"; break;
        case "Imperial Trader":           role = mark + "imptrader"; break;
		case "Cobra Mk IV Black":           role = mark + "cobra4"; break;
		case "Cobra S9":           role = mark + "cobras9"; break;
		case "Gnat":           role = mark + "gnat"; break;
		case "Bug":           role = mark + "bug"; break;
		case "Arachnid":           role = mark + "arachnid"; break;
		case "Monitor Mk II":           role = mark + "monitor2"; break;
		case "Griffin Mk I":           role = mark + "griffin1"; break;
        default:                    role = "pirate";
    }
    return role;
};

this.selectSystemsInRange = function(minRange, maxRange){
    // Create an array of systems suitable for missions. (called once on docking a seedy bar)
    var thisSystem = system.info;
    
    var targetSystems = SystemInfo.filteredSystems(this, function(other){ 
           return (other.systemID !== thisSystem.systemID) && (thisSystem.distanceToSystem(other) >= minRange) 
                                    && (thisSystem.distanceToSystem(other) <= maxRange && other.government < 6 && !system.sun.hasGoneNova); 
    }); 

    return targetSystems;
};

this.selectOneSystemInRange = function(targetSystems, minRange, maxRange){
    // select a system by distance from the pre-selected system list. (called for every generated mission)
    if (!targetSystems) return system.info; // current system
    var thisSystem = system.info;
    
    targetSystems = targetSystems.filter(function(other){ 
           return (other.systemID !== thisSystem.systemID) && (thisSystem.distanceToSystem(other) >= minRange) 
                                    && (thisSystem.distanceToSystem(other) <= maxRange); 
    }); 
    
    if (!targetSystems) return system.info; // current system
    
    var count = targetSystems.length;
    var targetSystem = targetSystems[Math.floor(Math.random()*count)];
    var i = 0;
    var route = System.infoForSystem(galaxyNumber, targetSystem.systemID).routeToSystem(thisSystem);
    while ((!route || !route.distance) && i<count){
        // log(this.name, "Found a system without a route to it: " + targetSystem.name + ". Selecting a new system");
        targetSystem = targetSystems[i++];  // Make sure there is a route, so no destinations in isolated areas are selected.
        route = System.infoForSystem(galaxyNumber, targetSystem.systemID).routeToSystem(thisSystem);
    }

    return targetSystem;
};

this.setupDockedVariables = function (){
    missionVariables.random_hits_seedy_spacebar_name = player.ship.dockedStation.displayName;
    // Below is the local bartender name and the consumption fee set.
    missionVariables.random_hits_billdue_name = expandDescription("[assassination_board_poster_name]"); 
    this.random_hits_billdue = (this.anyInt(21,49)/10).toFixed(1); // random decimal value as formatted string.
};

this.$restartHitPages = function (){
    this.showScreen = "firstPage";
    this.setupDockedVariables();
    this.missionOffers();
};

this.setupHitpages = function (){
    // create all the variables for the 3 different mission pages for later retrieval.
    this.hitpageNumber = 0; // reset number here.
    var count = Math.round(Math.random()*5);
    var name = "";
    if (0 < oolite.compareVersion("1.76")) for (var i=0; i<count; i++) name = expandDescription("[nom2]"); // mix up the random generator
    var startTime = Date.now();
    var maximumTime = 350; // time in msec.
    this.hitPageDate = clock.minutes;
    if (this.sets > 3) this.sets = 3; // maximum allowed is 3 sets. 3 sets means 9 mission pages but might give a long gap on slower systems.
    this.hitpageMaxNumber = 0;
    var targetSystems = this.selectSystemsInRange(5, 33);
    for (var j=0; j<this.sets; j++){
        for (var i=0; i<3; i++){
            var nr = i+3*j;
            var targetSystem = this.selectOneSystemInRange(targetSystems, 5+i*2, 21+i*4);
            var distance = system.info.distanceToSystem(targetSystem).toFixed(1);
            this.pages[nr].mark_system = targetSystem.name;
            this.pages[nr].mark_direction = "(at "+distance+" LY to the "+this.positionToDirection(targetSystem.coordinates)+")";
            this.pages[nr].assassination_board_poster_system = System.systemNameForID(Math.floor(Math.random()*256));
            if (0 < oolite.compareVersion("1.76")) // bad randomness is not fixed in 1.75 so we still need the next two lines.
            {
                count = Math.round(Math.random()*10);
                for (var k=0; k<count; k++) name = expandDescription("[nom2]"); // mix up the random generator
            }
            this.pages[nr].mark_gender = expandDescription("[mark_gender]");
            missionVariables.random_hits_mark_gender = this.pages[nr].mark_gender;// must be known later on.
            this.pages[nr].mark_ship_description = expandDescription("[mark_ship_description-"+(i+1)+"]");
                        
            this.pages[nr].mark_first_name = expandDescription("[mark_first_name-"+(i+1)+"]");
            this.pages[nr].mark_nick_name = expandDescription("[mark_nick_name-"+(i+1)+"]");
            this.pages[nr].mark_second_name = expandDescription("[mark_second_name-"+(i+1)+"]");
            this.pages[nr].mark_race_part1 = expandDescription("[mark_race_part1-"+(i+1)+"]");
            this.pages[nr].mark_race_part2 = expandDescription("[mark_race_part2-"+(i+1)+"]");
            missionVariables.random_hits_assassination_board_poster_system = this.pages[nr].assassination_board_poster_system; // needed below
            this.pages[nr].assassination_board_poster_title = expandDescription("[assassination_board_poster_title-"+(i+1)+"]");
            this.pages[nr].assassination_board_job_name = expandDescription("[assassination_board_job_name-"+(i+1)+"]");
            this.pages[nr].assassination_board_poster_surname = expandDescription("%N");
            this.pages[nr].assassination_board_poster_name = expandDescription("[assassination_board_poster_name-"+(i+1)+"]");
            this.pages[nr].assassination_board_address1 = expandDescription("[assassination_board_address1-"+(i+1)+"]");
            this.pages[nr].assassination_board_address2 = expandDescription("[assassination_board_address2-"+(i+1)+"]");
            this.pages[nr].assassination_board_part1 = expandDescription("[assassination_board_part1-"+(i+1)+"]");
            this.pages[nr].assassination_board_part2 = expandDescription("[assassination_board_part2-"+(i+1)+"]");
            this.pages[nr].assassination_board_part3 = expandDescription("[assassination_board_part3-"+(i+1)+"]");
            this.pages[nr].assassination_board_part4 = expandDescription("[assassination_board_part4-"+(i+1)+"]");
            this.pages[nr].assassination_board_part5 = expandDescription("[assassination_board_part5-"+(i+1)+"]");
            this.pages[nr].assassination_board_part6 = expandDescription("[assassination_board_part6-"+(i+1)+"]");
            this.pages[nr].assassination_board_part7 = expandDescription("[assassination_board_part7-"+(i+1)+"]");
            this.pages[nr].assassination_board_subject = expandDescription("[assassination_board_subject-"+(i+1)+"]");
            this.pages[nr].mark_ship = this.marks[i][Math.floor(Math.random()*this.marks[i].length)];

            //Random shipname for marks if oxp is present
            if (!worldScripts.randomshipnames) {
				this.pages[nr].mark_ship_name = this.pages[nr].mark_ship;
				this.pages[nr].mark_ship_ad_name = this.pages[nr].mark_ship;
			}
			else {
				var randomName = worldScripts.randomshipnames.$randomTraderName(player.ship);
				this.pages[nr].mark_ship_name = this.pages[nr].mark_ship + ": " + randomName;
				this.pages[nr].mark_ship_ad_name = this.pages[nr].mark_ship + ": '" + randomName + "'";
			}
            
            this.pages[nr].mark_fee = this.anyRoundedInt(350 + i*i*1650, 700+i*4150);
            // select marks by a JS array because that give a better random selection than those from desciptions.plist
            this.hitpageMaxNumber++;
            // log(this.name, "Created page "+this.hitpageMaxNumber+" after "+(Date.now() - startTime)+" msec.");
        }
        if (Date.now() - startTime > maximumTime){
           // we are busy for 0.5 (or 0.25) seconds. Don't generate more pages on slower machines because you get noticeable pauses
           // on docking and there is a risk of hitting the time-limiter introduced in Oolite 1.75.
           break;
           //j = 3;
        }
    }
    missionVariables.random_hits_show_galaxy = galaxyNumber+1;
};

this.readHitpage = function (nr){
    // fetch the previous stored page nr.
    nr--; // make zero based
    missionVariables.random_hits_mark_system = this.pages[nr].mark_system;
    missionVariables.random_hits_mark_direction = this.pages[nr].mark_direction;
    missionVariables.random_hits_mark_gender = this.pages[nr].mark_gender;
	missionVariables.random_hits_assassination_board_poster_title = this.pages[nr].assassination_board_poster_title;
	missionVariables.random_hits_assassination_board_job_name = this.pages[nr].assassination_board_job_name;
	missionVariables.random_hits_assassination_board_poster_surname = this.pages[nr].assassination_board_poster_surname;
	missionVariables.random_hits_assassination_board_poster_name = this.pages[nr].assassination_board_poster_name;
	missionVariables.random_hits_assassination_board_poster_system = this.pages[nr].assassination_board_poster_system;
    missionVariables.random_hits_assassination_board_subject =  this.pages[nr].assassination_board_subject;
	missionVariables.random_hits_assassination_board_address1 = this.pages[nr].assassination_board_address1;
	missionVariables.random_hits_assassination_board_address2 = this.pages[nr].assassination_board_address2;
	missionVariables.random_hits_assassination_board_part1 = this.pages[nr].assassination_board_part1;
	missionVariables.random_hits_mark_first_name = this.pages[nr].mark_first_name;
	missionVariables.random_hits_mark_nick_name = this.pages[nr].mark_nick_name;
	missionVariables.random_hits_mark_second_name = this.pages[nr].mark_second_name;
	missionVariables.random_hits_mark_race_part1 = this.pages[nr].mark_race_part1;
	missionVariables.random_hits_mark_race_part2 = this.pages[nr].mark_race_part2;
	missionVariables.random_hits_assassination_board_part2 = this.pages[nr].assassination_board_part2;
	missionVariables.random_hits_assassination_board_part3 = this.pages[nr].assassination_board_part3;
	missionVariables.random_hits_mark_ship_description = this.pages[nr].mark_ship_description;
	missionVariables.random_hits_mark_ship = this.pages[nr].mark_ship;
	missionVariables.random_hits_mark_ship_name = this.pages[nr].mark_ship_name;
	missionVariables.random_hits_mark_ship_ad_name = this.pages[nr].mark_ship_ad_name;
	missionVariables.random_hits_assassination_board_part4 = this.pages[nr].assassination_board_part4;
	missionVariables.random_hits_assassination_board_part5 = this.pages[nr].assassination_board_part5;
	missionVariables.random_hits_mark_fee = this.pages[nr].mark_fee;
	missionVariables.random_hits_assassination_board_part6 = this.pages[nr].assassination_board_part6;
	missionVariables.random_hits_assassination_board_part7 = this.pages[nr].assassination_board_part7;
};

this.positionToDirection = function(pos){
    var pos1 = system.info.coordinates;
    var pos2 = new Vector3D(pos);
    var dx, dy, x2, y2;
    
    dx = pos2.x - pos1.x;
    dy = pos2.y - pos1.y;
    x2 = Math.abs(dx);
    y2 = Math.abs(dy);
    
    if (x2 > 2.41421 * y2){  // x2 > y2 / Math.tan(Math.PI/8)
        return dx > 0? "east" : "west";
    }
    if (y2 > 1.732 * x2){
        return dy < 0? "north" : "south";
    }
    if (dx > 0){
        return dy < 0? "north-east" : "south-east";
    }else{
        return dy < 0? "north-west" : "south-west";
    }
};

this.updateStatus = function (){
        // Set the short description back to display current ranking following an completed mission.
        this.$unmarkSystem(missionVariables.random_hits_planetnumber);
        this.resetStoreVariables();
        this.setRank();
        this.updateTitle();
        if (missionVariables.random_hits_score <= 0){
            mission.setInstructionsKey("random_hits_shortdescription9");
        } else {
            mission.setInstructionsKey("random_hits_shortdescription3");
        }
        delete missionVariables.random_hits_status;
};

this.setRank = function (){
    var score = missionVariables.random_hits_score; // missionVariables are slow in processing;
    var rank;
    if (score <= 1) rank = expandDescription("[random_hits_rank1a]")
    else if (score === 2) rank = expandDescription("[random_hits_rank1b]")
    else if (score === 3) rank = expandDescription("[random_hits_rank1c]")
    else if (score === 4) rank = expandDescription("[random_hits_rank1d]")
    else if (score === 5) rank = expandDescription("[random_hits_rank1]")
    else if (score === 6) rank = expandDescription("[random_hits_rank2]")
    else if (score === 7) rank = expandDescription("[random_hits_rank3]")
    else if (score === 8) rank = expandDescription("[random_hits_rank4]")
    else if (score === 9) rank = expandDescription("[random_hits_rank5]")
    else if (score === 10) rank = expandDescription("[random_hits_rank6]")
    else rank = expandDescription("[random_hits_rank"+(this.modulo(score-1,10)+1)+"]");
    missionVariables.random_hits_currentrank = rank;
};

this.updateTitle = function (){
    var score = missionVariables.random_hits_score;
    var title;
    if (score < 1) title = "random_hits_huntertitle0"
    else if (score < 5) title = "random_hits_huntertitle1"
    else if (score < 101) title = "random_hits_huntertitle"+Math.floor((score-1)/10+2)// one title higher for every 10 points
    else if (score < 150) title = "random_hits_huntertitle12"
    else title = "random_hits_huntertitle13";
    missionVariables.random_hits_playertitle = expandDescription("["+title+"]");
};

this.decreaseCriminalHateBy = function (hate){
    missionVariables.random_hits_criminal_hate_score -= hate;
    if (missionVariables.random_hits_criminal_hate_score < 0){
        missionVariables.random_hits_criminal_hate_score = 0;
    }
};

this.testRevenge = function(){
    //  Roll dice to see if Victim's relatives will take revenge following a hit! 
    if (missionVariables.random_hits_revenge === "YES") return;
    if (missionVariables.random_hits_revenge === "NO" || missionVariables.random_hits_revenge === "KILLED"){
			missionVariables.random_hits_revenge = (Math.random() < 0.33) ? "YES" : "NO";
			missionVariables.random_hits_striketime  = expandDescription("[random_hits_revenge_dice2]");
			missionVariables.random_hits_timer = 0;
			missionVariables.random_hits_criminal_hate_score += this.anyInt(6,10);
            delete missionVariables.random_hits_revenge_warned;
    }    
};

this.evaluateRevengeStatus = function (){
    log(this.name, expandDescription("(Hatred : [mission_random_hits_criminal_hate_score]. Events Timer : [mission_random_hits_events_timer]. Dockings : [mission_random_hits_dockings_number]. Events Number : [mission_random_hits_events_number]. Special Mission Timer : [mission_random_hits_special_mission_timer]. Revenge Status : [mission_random_hits_revenge]. Time to Strike : [mission_random_hits_striketime]. Revenge Jumps Made : [mission_random_hits_timer]. Special Mission : [mission_random_hits_special_mission]. Master Timer : [mission_random_hits_events_timer]. Dice Roll : [mission_random_hits_events_dice])"));
    var hate = missionVariables.random_hits_criminal_hate_score;
    var rank;
    if (hate < 5) rank = "an innocent fighter"
    else if (hate < 25) rank = "an annoyance"
    else if (hate < 50) rank = "a "+expandDescription("[random_hits_notwelcome_para3e]")+" pilot"
    else if (hate < 70) rank = "a target that must be "+expandDescription("[random_hits_complete2]")
    else if (hate < 90) rank = "on their top ten hit list of "+expandDescription("[random_hits_notwelcome_para3e]")+" pilots"
    else rank = "their top priority for being "+expandDescription("[random_hits_complete2]");
    missionVariables.random_hits_triad_rank = rank;
};

this.broadcastBigBossCount = function (){
    // This function is also called by shipscripts so don't change its name !! 
    if (!this.bigBossAttack) return;
    this.countBigBosses();
    if (!player.ship.docked){
        if (missionVariables.random_hits_battle_boss_number === 0 && missionVariables.random_hits_battle_fighter_number === 0){
            player.commsMessage(expandDescription("[random_hits_big_boss_inform_over]"), 10);
        } else {
            player.consoleMessage(expandDescription("[random_hits_big_boss_inform]"), 6);
        }
    }
    delete missionVariables.random_hits_describe_player_kill_count_up;
    delete missionVariables.random_hits_describe_boss_number;
    delete missionVariables.random_hits_describe_fighter_number;
};

this.countBigBosses = function (){
    missionVariables.random_hits_battle_boss_number = system.countShipsWithPrimaryRole("random_hits_big_boss");
    missionVariables.random_hits_battle_fighter_number = system.countShipsWithPrimaryRole("random_hits_big_boss_fighter");
    switch(missionVariables.random_hits_battle_boss_number){
        case 0: missionVariables.random_hits_describe_boss_number = expandDescription("[random_hits_big_boss_inform_no_boss]");
                break;
        case 1: missionVariables.random_hits_describe_boss_number = expandDescription("[random_hits_big_boss_inform_one_boss]");
                break;
        default: missionVariables.random_hits_describe_boss_number = expandDescription("[random_hits_big_boss_inform_many_boss]");
    }
    switch(missionVariables.random_hits_battle_fighter_number){
        case 0: missionVariables.random_hits_describe_fighter_number = expandDescription("[random_hits_big_boss_inform_no_fighters]");
                break;
        case 1: missionVariables.random_hits_describe_fighter_number = expandDescription("[random_hits_big_boss_inform_one_fighters]");
                break;
        default: missionVariables.random_hits_describe_fighter_number = expandDescription("[random_hits_big_boss_inform_many_fighters]");
    }
    switch(missionVariables.random_hits_total_player_boss_kills){
        case 0: missionVariables.random_hits_describe_player_kill_count_up = expandDescription("[random_hits_big_boss_inform_no_kills]");
                break;
        case 1: missionVariables.random_hits_describe_player_kill_count_up = expandDescription("[random_hits_big_boss_inform_one_kill]");
                break;
        default: missionVariables.random_hits_describe_player_kill_count_up = expandDescription("[random_hits_big_boss_inform_many_kills]");
    }
};

this.setThargoidMessages = function (){
    missionVariables.random_hits_thargoid_number = system.countShipsWithPrimaryRole("thargoid");
    missionVariables.random_hits_thargoid_number_close = system.shipsWithPrimaryRole("thargoid", player.ship.dockedStation, 25E3).length;
    if (missionVariables.random_hits_thargoid_number_close === 0){
        this.thargoidAttack = false;
        return "barThargoidAttackEnded";
    } else {
        return "barThargoidAttackInProgress";
    }
};

this.setMiningpodTheftMessages = function (){
    missionVariables.random_hits_miningpodThief_number = system.shipsWithPrimaryRole("random_hits_miningpodThief", player.ship.dockedStation, 25E3).length;
    if (missionVariables.random_hits_miningpodThief_number === 0){
        this.miningpodTheft = false;
        return "miningpodTheftEnded";
    } else {
        return "miningpodTheftInProgress";
    }
};

this.setUpThargoidRewards = function (group){
    // Insert a counter in the script of all groupmember death-actions.
    for (var i = 0; i < group.count; i++){
        var thargoid = group.ships[i];
        if (thargoid.script.shipDied) thargoid.script.randomHitsOriginal = thargoid.script.shipDied;
        thargoid.script.shipDied = function (who, why){ 
            if (who && who.isPlayer){
                missionVariables.random_hits_player_thargoid_kills++;
            }
            if (this.randomHitsOriginal) this.randomHitsOriginal(who, why);
        } 
    }
};

this.setUpMiningpodThief = function (group){
    // convert plain pirates in to miningpod thieves.
    group.name = "Cargo Thieves";
    group.leader = group.ships[0]; // select a random leader
    for (var i = 0; i < group.count; i++){
        var thief = group.ships[i];
        thief.setScript("oolite-random-hits-miningpod-thief.js");
        thief.switchAI("randomhitsminingpodthiefAI.plist");
        thief.primaryRole = "random_hits_miningpodThief";
        thief.bounty = 0; // start clean to not alert the police.
        if (thief.equipmentStatus("EQ_FUEL_SCOOPS") === "EQUIPMENT_UNAVAILABLE") thief.awardEquipment("EQ_FUEL_SCOOPS");
    }
    return group.count;
};

this.setUpBigBossSpacelaneFighter = function (group){
    // convert plain big_boss_fighters (or plain pirates) to their spacelane variant. 
    // these ships are currently unused in the code.
    for (var i = 0; i < group.count; i++){
        var fighter = group.ships[i];
        fighter.setScript("oolite-random-hits-bigboss-fighter.js");
        fighter.switchAI("randomhitsbigbossAI.plist");
        fighter.primaryRole = "random_hits_big_boss_fighter_spacelane";
    }
    return group.count;
};

this.setBigBossMessages = function (){
    missionVariables.random_hits_battle_boss_number = system.countShipsWithPrimaryRole("random_hits_big_boss");
    missionVariables.random_hits_battle_fighter_number = system.countShipsWithPrimaryRole("random_hits_big_boss_fighter");
    log(this.name, "The player killed; " + missionVariables.random_hits_total_player_boss_kills + " bosses");
    
    // If the battle is over do a body count and set up strings to comment on this.
    if (missionVariables.random_hits_battle_boss_number === 0 && missionVariables.random_hits_battle_fighter_number === 0){
        var total_player_boss_kills = missionVariables.random_hits_total_player_boss_kills;
        var kills, prize;
        if (total_player_boss_kills === 0){
            kills = expandDescription("[random_hits_security_bar_text_describe0]");
            prize = expandDescription("[random_hits_big_boss_prize_npc]");
        } else if (total_player_boss_kills === 1){
            kills = expandDescription("[random_hits_security_bar_text_describe1]");
            prize = expandDescription("[random_hits_big_boss_prize_npc1]");
        } else if (total_player_boss_kills < 4){
            kills = expandDescription("[random_hits_total_player_big_boss_kills]");
            prize = expandDescription("[random_hits_big_boss_prize_npc2]");
        } else if (total_player_boss_kills < 7){
            kills = expandDescription("[random_hits_total_player_big_boss_kills]");
            prize = expandDescription("[random_hits_big_boss_prize_npc2]");
        } else {
            kills = expandDescription("[random_hits_total_player_big_boss_kills]");
            prize = expandDescription("[random_hits_big_boss_prize_player]");
        }
        missionVariables.random_hits_describe_total_player_kills = kills;
        missionVariables.random_hits_describe_prize = prize;

        if (total_player_boss_kills < 8){
            missionVariables.random_hits_total_boss_kills = expandDescription("[random_hits_unseen_kills_number]");
        }

        if (missionVariables.random_hits_hunter_kill_score === 0){
            missionVariables.random_hits_security_bar_text_describe_hunter_kills = expandDescription("[random_hits_security_bar_text_describe_hunters0_alt]");
            missionVariables.random_hits_security_bar_text_describe_hunter_kills_pre = expandDescription("[random_hits_security_bar_text_describe_hunters0_pre_none]");
        } else if (missionVariables.random_hits_hunter_kill_score === 1){
            missionVariables.random_hits_security_bar_text_describe_hunter_kills = expandDescription("[random_hits_security_bar_text_describe_hunters1_alt]");
            missionVariables.random_hits_security_bar_text_describe_hunter_kills_pre = expandDescription("[random_hits_security_bar_text_describe_hunters0_pre_some]");
        } else {
            missionVariables.random_hits_security_bar_text_describe_hunter_kills = expandDescription("[random_hits_security_bar_text_describe_hunters2_alt]");
            missionVariables.random_hits_security_bar_text_describe_hunter_kills_pre = expandDescription("[random_hits_security_bar_text_describe_hunters0_pre_some]");
        }
        
        // Following the battle, check if any further bounty payments are due.
        this.calculateBigBossPayments();
        this.bigBossAttack = false;
        return "barAttackEnded";
        
    } else {
        var boss_number = missionVariables.random_hits_battle_boss_number;
        if (boss_number > 2) boss_number = 2;  // boss_number is 0, 1 or 2.
        missionVariables.random_hits_describe_boss_left = expandDescription("[random_hits_security_bar_text_boss_describe"+boss_number+"]");

        var fighter_number = missionVariables.random_hits_battle_fighter_number;
        if (fighter_number > 2) fighter_number = 2;
        missionVariables.random_hits_describe_fighter_left = expandDescription("[random_hits_security_bar_text_fighter_describe"+fighter_number+"]");

        this.calculateBigBossPayments();
        
        // If the Player docks with a battle in progress, count up his kills if any.
        var kill_score = missionVariables.random_hits_hunter_kill_score;
        if (kill_score > 2) kill_score = 2;
        missionVariables.random_hits_security_bar_text_describe_hunter_kills = 
                                        expandDescription("[random_hits_security_bar_text_describe_hunters"+kill_score+"]");
    }
    return "barAttackInProgress";
};

this.clearBigBossDescriptions = function (){
    var deletions = ["criminal_kill_payment", "security_bar_text_describe_kills", "describe_boss_left", "describe_fighter_left", 
    "security_bar_text_describe_payment", "security_bar_text_describe_hunter_kills", "security_bar_text_describe_hunter_kills_pre", 
    "battle_boss_number", "battle_fighter_number"];
    for (var i=0; i<deletions.length; i++) delete missionVariables["random_hits_"+deletions[i]];
    missionVariables.random_hits_criminal_kill_score = 0;
};

this.calculateBigBossPayments = function (){
    var score = missionVariables.random_hits_criminal_kill_score;
    var describe_kills, describe_payment, payment;
    if (score === 0){
        describe_kills = expandDescription("[random_hits_security_bar_text_describe0]");
        describe_payment = expandDescription("[random_hits_security_bar_text_describe_payment0]");
        payment = 0;
    } else if (score === 1){
        describe_kills = expandDescription("[random_hits_security_bar_text_describe1]");
        describe_payment = expandDescription("[random_hits_security_bar_text_describe_payment1]");
        payment = this.anyRoundedInt(250, 450);
    } else if (score < 4){
        describe_kills = expandDescription("[random_hits_security_bar_text_describe2]");
        describe_payment = expandDescription("[random_hits_security_bar_text_describe_payment1]");
        payment = this.anyRoundedInt(450, 600);
    } else if (score < 8){
        describe_kills = expandDescription("[random_hits_security_bar_text_describe2]");
        describe_payment = expandDescription("[random_hits_security_bar_text_describe_payment1]");
        payment = this.anyRoundedInt(600, 800);
    } else {
        describe_kills = expandDescription("[random_hits_security_bar_text_describe2]");
        describe_payment = expandDescription("[random_hits_security_bar_text_describe_payment1]");
        payment = this.anyRoundedInt(850, 1050);
    }
    missionVariables.random_hits_security_bar_text_describe_kills = describe_kills;
    missionVariables.random_hits_security_bar_text_describe_payment = describe_payment;
    missionVariables.random_hits_criminal_kill_payment = payment;
};

this.$markSystem = function (ID)
{
    if (0 < oolite.compareVersion("1.77"))
    {
        mission.markSystem(ID);
    }
    else
    {
        mission.markSystem({
                   system: ID,
                   name: "Random_Hits",
                   markerColor: "yellowColor",
                   markerScale: 1.0,
                   markerShape: "MARKER_SQUARE"
        });
    }
    this.systemUnmarked = false;
}

this.$unmarkSystem = function (ID)
{
    if (this.systemUnmarked) return; // don't unmark twice.
    if (0 < oolite.compareVersion("1.77"))
    {
        mission.unmarkSystem(ID);
    }
    else
    {
        var ok = mission.unmarkSystem({system: ID, name: "Random_Hits"})
        if (!ok) mission.unmarkSystem(ID); // must be an old legacy marker.
    }
    this.systemUnmarked = true;
}
